// Copyright 1998-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

// This launcher application is a hack to allow us to bundle frameworks inside our app-wrapper, solving the versionitis problems associated with releasing different apps (OmniWeb, OmniPDF, etc.) all of which use the same set of frameworks.  The basic idea is we create a launcher application that just adds its wrapper directory to the DYLD_FRAMEWORK_PATH environment variable, then launches the real application.

// When building this for product, you should copy the real application and all its associated frameworks into the Resources subdirectory of the launcher application.  Also, make sure that the CustomInfo.plist here is in synch with the one in the real application.  (We should probably just make it a symbolic link.)  Then you can install the launcher wrapper anywhere (e.g. /Local/Applications) and it will run without any additional frameworks needing to be installed.

// Turn off OBASSERT, since it relies on code in OmniBase, and we don't actually link against OmniBase (otherwise we'd need ourselves to launch)
#define OMNIMAKE_FORCE_ASSERTIONS_OFF
#import <OmniBase/OmniBase.h>

#import <unistd.h>
#import <stdlib.h>
#import <string.h>
#import <sys/param.h>
#import <stdio.h>
#import <stdlib.h>

RCS_ID("$Header: /Network/Developer/Source/CVS/OmniGroup/OmniWebLauncher/OmniWeb_main.m,v 1.4 1999/04/19 20:57:22 kc Exp $");

extern char **environ;

#import "setenv.h" // Setenv isn't a standard function on OPENSTEP/Mach

#ifdef YELLOW_BOX
#define DEFAULT_SYSTEM_FRAMEWORK_PATH "/Local/Library/Frameworks:/Network/Library/Frameworks:/System/Library/Frameworks"
#else
#define DEFAULT_SYSTEM_FRAMEWORK_PATH "/LocalLibrary/Frameworks:/NextLibrary/Frameworks"
#endif

int main(int argc, const char *argv[]) {
    char *currentFrameworkPath;
    char newFrameworkPath[8192];
    const char *pathToLauncherBinary;
    const char *pointerToFinalSlashInPathToLauncherBinary;
    char pathToLauncherAppWrapper[MAXPATHLEN];
    char pathToRealApplicationBinary[MAXPATHLEN];
    const char *applicationName;

    // Note:  Yes, this code would be cleaner in Objective C.  However, that would mean we'd have to start up the Objective C run-time, and since we're just going to terminate as soon as possible we really don't want this to take longer than necessary.  (Also, the Objective C run-time probably has some side effects with respect to our mach ports which we don't want to trigger.)

    // The path to the launcher binary (that's us, dude!) is in argv[0]
    OBASSERT(argc > 0);
    pathToLauncherBinary = argv[0];

    // Calculate the application name and the path to its wrapper
    pointerToFinalSlashInPathToLauncherBinary = rindex(pathToLauncherBinary, '/');
    if (!pointerToFinalSlashInPathToLauncherBinary) {
        // No slashes in the path to the launcher binary

        // Our application name is simply the path
        applicationName = pathToLauncherBinary;

        // Our wrapper had better be the current working directory.
        getwd(pathToLauncherAppWrapper);
    } else {
        // Our application name follows that final slash
        applicationName = pointerToFinalSlashInPathToLauncherBinary + 1;

        // The (possibly relative) path to the app wrapper is everything up to that final slash
        strcpy(pathToLauncherAppWrapper, pathToLauncherBinary);
        pathToLauncherAppWrapper[pointerToFinalSlashInPathToLauncherBinary - pathToLauncherBinary] = '\0'; // truncate at the final slash

        if (pathToLauncherAppWrapper[0] == '/') {
            // The app wrapper path is already an absolute path, just use it
        } else {
            char originalWorkingDirectory[MAXPATHLEN];

            // Turn the app wrapper path into a relative path.  This would sure be easy with NSString's path utilities, but...

            // Save the original working directory
            getwd(originalWorkingDirectory);
            // chdir() accepts relative paths
            chdir(pathToLauncherAppWrapper);
            // and getwd() returns absolute paths
            getwd(pathToLauncherAppWrapper);
            // Restore the original working directory
            chdir(originalWorkingDirectory);
        }
    }
    // We've now calculated our application name, and the absolute path to our app wrapper
    OBASSERT(applicationName != NULL && applicationName[0] != '\0');
    OBASSERT(pathToLauncherAppWrapper[0] == '/');

    // Create a new framework path, so it will search for frameworks within our app wrapper before it looks for them anywhere else.

    // Start with the path to our app wrapper's Resources subdirectory
    sprintf(newFrameworkPath, "%s/Resources", pathToLauncherAppWrapper);
    strcat(newFrameworkPath, ":");

    // Append the standard framework path
    if ((currentFrameworkPath = getenv("DYLD_FRAMEWORK_PATH"))) {
        // There's already a framework path, append it to the new path
        strcat(newFrameworkPath, currentFrameworkPath);
    } else {
        const char *homeDirectory;

        // There's no framework path currently set, so we should append the standard framework path

        // Append $HOME/Library/Frameworks to the path
        if ((homeDirectory = getenv("HOME"))) {
            strcat(newFrameworkPath, homeDirectory);
            strcat(newFrameworkPath, "/Library/Frameworks:");
        }

        // Append the default system framework path to the new path
        strcat(newFrameworkPath, DEFAULT_SYSTEM_FRAMEWORK_PATH);
    }

    // Set the DYLD_FRAMEWORK_PATH environment variable to use our new path
    setenv("DYLD_FRAMEWORK_PATH", newFrameworkPath, 1);

    // Launch the real application

    // It should live in the launcher's appWrapper's Resources directory under the name <applicationName>.app/<applicationName>
    sprintf(pathToRealApplicationBinary, "%s/Resources/%s.app/%s", pathToLauncherAppWrapper, applicationName, applicationName);

    // Let the user know what we're doing
    fprintf(stderr, "Launching %s...\n", pathToRealApplicationBinary);

    // We'll be passing the real application the exact same arguments we received, except that we'll pass it the path to itself in argv[0] rather than the path to us.
    argv[0] = pathToRealApplicationBinary;

    // Execute the real application.  If successful, this call does not return.  (It doesn't fork off a process, it launches the other application in place of the current application within the current process.)
    execve(pathToRealApplicationBinary, argv, environ);

    // This code does not get called in normal operation:  if execve() succeeds, it doesn't return, so in this case execve() must have failed.

    // Print the error message associated with the current error condition
    perror(pathToRealApplicationBinary);

    // Return an error status.
    return 1;
}
